#ifndef ccbcontrolserver_h
#define ccbcontrolserver_h

#include "ccbserverlink.h"  /* Get the CCBServerLink typedef */

/*-----------------------------------------------------------------------*
 * This file defines control-link communications interface between       *
 * a CCB server and a CCB manager.                                       *
 *-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*
 *              The server interface to the control link                 *
 *-----------------------------------------------------------------------*
 *
 * CCBControlServer objects manage the control link of the CCB server.
 */
typedef struct CCBControlServer CCBControlServer;

CCBControlServer *new_CCBControlServer(CCBServerLink *server);
CCBControlServer *del_CCBControlServer(CCBControlServer *cs);

/*
 * Attempt to send all queued control messages to the manager.
 */
int ccb_cs_send_control(CCBControlServer *cs);

/*
 * Read up to one command message from the CCB server control socket, and
 * when a complete command-message has been read, deliver it to the
 * application by calling the appropriate callback function.
 */
int ccb_cs_read_control(CCBControlServer *cs);

/*
 * Attach the control server to a new socket (ie. a new manager connected).
 */
int ccb_cs_attach(CCBControlServer *cs, int fd);

/*
 * Detach the control server from the control socket.
 */
void ccb_cs_detach(CCBControlServer *cs);

/*
 * Enumerate the states that the control server can be in. Note
 * that when the server isn't down it is always receptive to
 * incoming commands, so this isn't enumerated.
 */
typedef enum {
  CCB_CONTROL_SERVER_DOWN,/* Their is currently no control link */
  CCB_CONTROL_SERVER_IDLE,/* No messages are currently queued */
  CCB_CONTROL_SERVER_BUSY /* The server is in the midst of sending a message */
} CCBControlServerStatus;

/*
 * Query the current status of the control server.
 */
CCBControlServerStatus ccb_cs_status(CCBControlServer *cs);

/*
 * Reset both the control server and the CCB to power-on defaults.
 */
int ccb_cs_reset(CCBControlServer *cs);

/*
 * Abort any ongoing scan or intra-scan, install the current scan
 * configuration, then start an intra-scan.
 */
int ccb_cs_intra_scan(CCBControlServer *cs, unsigned long scan);

/*
 * Abort any ongoing scan or intra-scan, install the current scan
 * configuration, then start a dump-mode intra-scan.
 */
int ccb_cs_dump_scan(CCBControlServer *cs, unsigned long scan,
		     unsigned short adc, unsigned long samples,
		     unsigned long frames);

#endif
